/*
 * error.c -- error handler
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "curslib.h"
#include "error.h"


#define MAX_ENV_NEST_DEPTH	10


static jmp_buf *environments[MAX_ENV_NEST_DEPTH];
static int currentEnvironment = -1;

static char *exceptionStrings[] = {
  "No exception",
  "A bus exception",
  "An address exception",
  "An instruction exception",
  "A divide exception"
};


char *exceptionToString(int exception) {
  if (exception < 0 || exception >= NUM_EXCEPTIONS) {
    error("exception number out of bounds");
  }
  return exceptionStrings[exception];
}


void throwException(int exception) {
  if (currentEnvironment < 0) {
    error("exception %d thrown while no environment active", exception);
  }
  longjmp(*environments[currentEnvironment], exception);
}


void pushEnvironment(jmp_buf *environment) {
  if (currentEnvironment == MAX_ENV_NEST_DEPTH - 1) {
    error("too many environments active");
  }
  currentEnvironment++;
  environments[currentEnvironment] = environment;
}


void popEnvironment(void) {
  if (currentEnvironment < 0) {
    error("cannot pop environment - none active");
  }
  currentEnvironment--;
}


void error(char *fmt, ...) {
  va_list ap;

  cExit();
  va_start(ap, fmt);
  fprintf(stderr, "Error: ");
  vfprintf(stderr, fmt, ap);
  fprintf(stderr, "\n");
  va_end(ap);
  exit(1);
}
