/*
 * instr.h -- instruction encoding
 */


#ifndef _INSTR_H_
#define _INSTR_H_


#include "common.h"


#define OPCODE(x)	(((x) >> 24) & MASK(8))
#define DST(x)		(((x) >> 16) & MASK(8))
#define SRC1(x)		(((x) >>  8) & MASK(8))
#define SRC2(x)		(((x) >>  0) & MASK(8))
#define LIT16(x)	(((x) >>  0) & MASK(16))
#define LIT24(x)	(((x) >>  0) & MASK(24))


#define I_I8I8I8	0	/* three 8 bit immeds */
#define I_RRR		1	/* three regs */
#define I_RRI8		2	/* two regs, one 8 bit immed */
#define I_RI8R		3	/* one reg, one 8 bit immed, one reg */
#define I_RI8I8		4	/* one reg, two 8 bit immeds */
#define I_I8RR		5	/* one 8 bit immed, two regs */
#define I_I8RI8		6	/* one 8 bit immed, one reg, one 8 bit immed */
#define I_RI16		7	/* one reg, one 16 bit immed */
#define I_I8I16		8	/* one 8 bit immed, one 16 bit immed */
#define I_RF16		9	/* one reg, one 16 bit forward offset */
#define I_RB16		10	/* one reg, one 16 bit backward offset */
#define I_F24		11	/* one 24 bit forward offset */
#define I_B24		12	/* one 24 bit backward offset */
#define I_RZ8S8		13	/* one reg, 8 bit zero, 8 bit special */
#define I_S8Z8R		14	/* 8 bit special, 8 bit zero, one reg */
#define I_S8Z8I8	15	/* 8 bit special, 8 bit zero, 8 bit immed */


typedef struct {
  char *name;
  int type;
  byte opcode;
} Instr;


extern Instr instrTbl[];
extern Instr *instrCodeTbl[];


void initInstrTable(void);
Instr *lookupInstr(char *name);


#endif /* _INSTR_H_ */
