/*
 * main.c -- main program
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "curslib.h"
#include "error.h"
#include "command.h"
#include "greg.h"
#include "instr.h"
#include "cpu.h"
#include "mmu.h"
#include "memory.h"
#include "timer.h"
#include "term.h"
#include "disk.h"


static void usage(char *myself) {
  fprintf(stderr, "Usage: %s [<disk image>]\n", myself);
  exit(1);
}


int main(int argc, char *argv[]) {
  char *line;

  if (argc != 1 && argc != 2) {
    usage(argv[0]);
  }
  printf("GIMMIX Simulator started\n");
  cInit();
  cPrintf("GIMMIX Simulator started\n");
  initGregTable();
  initInstrTable();
  timerInit();
  termInit();
  if (argc == 2) {
    diskInit(argv[1]);
  } else {
    diskInit(NULL);
  }
  memoryInit();
  mmuInit();
  cpuInit();
  while (1) {
    line = cGetLine("GIMMIX > ");
    if (*line == '\0') {
      break;
    }
    cAddHist(line);
    if (execCommand(line)) {
      break;
    }
  }
  cpuExit();
  mmuExit();
  memoryExit();
  timerExit();
  termExit();
  diskExit();
  cPrintf("GIMMIX Simulator finished\n");
  cExit();
  printf("GIMMIX Simulator finished\n");
  return 0;
}
