/*
 * memory.c -- main memory simulation
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "curslib.h"
#include "error.h"
#include "cpu.h"
#include "memory.h"
#include "term.h"
#include "disk.h"


static byte *mainMemory;

static byte rom[ROM_SIZE] = {
  #include "rom/current.rom"
};


octa memoryRead64(octa pAddr) {
  octa data;

  if (pAddr.h == 0 &&
      pAddr.l <= MEMORY_SIZE - 8) {
    data.h = ((tetra) *(mainMemory + pAddr.l + 0)) << 24 |
             ((tetra) *(mainMemory + pAddr.l + 1)) << 16 |
             ((tetra) *(mainMemory + pAddr.l + 2)) <<  8 |
             ((tetra) *(mainMemory + pAddr.l + 3)) <<  0;
    data.l = ((tetra) *(mainMemory + pAddr.l + 4)) << 24 |
             ((tetra) *(mainMemory + pAddr.l + 5)) << 16 |
             ((tetra) *(mainMemory + pAddr.l + 6)) <<  8 |
             ((tetra) *(mainMemory + pAddr.l + 7)) <<  0;
    return data;
  }
  if (pAddr.h == ROM_BASE_H &&
      pAddr.l >= ROM_BASE_L &&
      pAddr.l <= ROM_BASE_L + ROM_SIZE - 8) {
    data.h = ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 0)) << 24 |
             ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 1)) << 16 |
             ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 2)) <<  8 |
             ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 3)) <<  0;
    data.l = ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 4)) << 24 |
             ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 5)) << 16 |
             ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 6)) <<  8 |
             ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 7)) <<  0;
    return data;
  }
  if ((pAddr.h & IO_CLASS_MASK) == TERM_CLASS) {
    data = termRead(pAddr.h & IO_NUMBER_MASK, pAddr.l);
    return data;
  }
  if ((pAddr.h & IO_CLASS_MASK) == DISK_CLASS) {
    data = diskRead(pAddr.h & IO_NUMBER_MASK, pAddr.l);
    return data;
  }
  throwException(BUS_EXCEPTION);
  /* not reached */
  data.h = 0;
  data.l = 0;
  return data;
}


tetra memoryRead32(octa pAddr) {
  tetra data;

  if (pAddr.h == 0 &&
      pAddr.l <= MEMORY_SIZE - 4) {
    data = ((tetra) *(mainMemory + pAddr.l + 0)) << 24 |
           ((tetra) *(mainMemory + pAddr.l + 1)) << 16 |
           ((tetra) *(mainMemory + pAddr.l + 2)) <<  8 |
           ((tetra) *(mainMemory + pAddr.l + 3)) <<  0;
    return data;
  }
  if (pAddr.h == ROM_BASE_H &&
      pAddr.l >= ROM_BASE_L &&
      pAddr.l <= ROM_BASE_L + ROM_SIZE - 4) {
    data = ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 0)) << 24 |
           ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 1)) << 16 |
           ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 2)) <<  8 |
           ((tetra) *(rom + (pAddr.l - ROM_BASE_L) + 3)) <<  0;
    return data;
  }
  throwException(BUS_EXCEPTION);
  /* not reached */
  data = 0;
  return data;
}


wyde memoryRead16(octa pAddr) {
  wyde data;

  if (pAddr.h == 0 &&
      pAddr.l <= MEMORY_SIZE - 2) {
    data = ((wyde) *(mainMemory + pAddr.l + 0)) << 8 |
           ((wyde) *(mainMemory + pAddr.l + 1)) << 0;
    return data;
  }
  if (pAddr.h == ROM_BASE_H &&
      pAddr.l >= ROM_BASE_L &&
      pAddr.l <= ROM_BASE_L + ROM_SIZE - 2) {
    data = ((wyde) *(rom + (pAddr.l - ROM_BASE_L) + 0)) << 8 |
           ((wyde) *(rom + (pAddr.l - ROM_BASE_L) + 1)) << 0;
    return data;
  }
  throwException(BUS_EXCEPTION);
  /* not reached */
  data = 0;
  return data;
}


byte memoryRead8(octa pAddr) {
  byte data;

  if (pAddr.h == 0 &&
      pAddr.l <= MEMORY_SIZE - 1) {
    data = ((byte) *(mainMemory + pAddr.l + 0)) << 0;
    return data;
  }
  if (pAddr.h == ROM_BASE_H &&
      pAddr.l >= ROM_BASE_L &&
      pAddr.l <= ROM_BASE_L + ROM_SIZE - 1) {
    data = ((byte) *(rom + (pAddr.l - ROM_BASE_L) + 0)) << 0;
    return data;
  }
  throwException(BUS_EXCEPTION);
  /* not reached */
  data = 0;
  return data;
}


void memoryWrite64(octa pAddr, octa data) {
  if (pAddr.h == 0 &&
      pAddr.l <= MEMORY_SIZE - 8) {
    *(mainMemory + pAddr.l + 0) = (byte) (data.h >> 24);
    *(mainMemory + pAddr.l + 1) = (byte) (data.h >> 16);
    *(mainMemory + pAddr.l + 2) = (byte) (data.h >>  8);
    *(mainMemory + pAddr.l + 3) = (byte) (data.h >>  0);
    *(mainMemory + pAddr.l + 4) = (byte) (data.l >> 24);
    *(mainMemory + pAddr.l + 5) = (byte) (data.l >> 16);
    *(mainMemory + pAddr.l + 6) = (byte) (data.l >>  8);
    *(mainMemory + pAddr.l + 7) = (byte) (data.l >>  0);
    return;
  }
  if ((pAddr.h & IO_CLASS_MASK) == TERM_CLASS) {
    termWrite(pAddr.h & IO_NUMBER_MASK, pAddr.l, data);
    return;
  }
  if ((pAddr.h & IO_CLASS_MASK) == DISK_CLASS) {
    diskWrite(pAddr.h & IO_NUMBER_MASK, pAddr.l, data);
    return;
  }
  throwException(BUS_EXCEPTION);
}


void memoryWrite32(octa pAddr, tetra data) {
  if (pAddr.h == 0 &&
      pAddr.l <= MEMORY_SIZE - 4) {
    *(mainMemory + pAddr.l + 0) = (byte) (data >> 24);
    *(mainMemory + pAddr.l + 1) = (byte) (data >> 16);
    *(mainMemory + pAddr.l + 2) = (byte) (data >>  8);
    *(mainMemory + pAddr.l + 3) = (byte) (data >>  0);
    return;
  }
  throwException(BUS_EXCEPTION);
}


void memoryWrite16(octa pAddr, wyde data) {
  if (pAddr.h == 0 &&
      pAddr.l <= MEMORY_SIZE - 2) {
    *(mainMemory + pAddr.l + 0) = (byte) (data >> 8);
    *(mainMemory + pAddr.l + 1) = (byte) (data >> 0);
    return;
  }
  throwException(BUS_EXCEPTION);
}


void memoryWrite8(octa pAddr, byte data) {
  if (pAddr.h == 0 &&
      pAddr.l <= MEMORY_SIZE - 1) {
    *(mainMemory + pAddr.l + 0) = (byte) (data >> 0);
    return;
  }
  throwException(BUS_EXCEPTION);
}


void memoryReset(void) {
  int i;

  cPrintf("Resetting Memory...\n");
  for (i = 0; i < MEMORY_SIZE; i++) {
    mainMemory[i] = rand();
  }
}


void memoryInit(void) {
  mainMemory = malloc(MEMORY_SIZE);
  if (mainMemory == NULL) {
    error("cannot allocate main memory");
  }
  memoryReset();
}


void memoryExit(void) {
  free(mainMemory);
}
