/*
 * prf.c -- kernel printf and friends
 */


#include "../h/param.h"
#include "../h/buf.h"
#include "../dev/kl.h"
#include "../conf/mch.h"
#include "alloc.h"
#include "prf.h"


/*
 * Scaled down version of C library printf.
 * Only %s %u %d %o %x %D are recognized.
 * Used to print diagnostic information
 * directly on the console.
 * Since it is not interrupt driven,
 * all system activities are pretty
 * much suspended.
 * Printf should not be used for chit-chat.
 */
void printf(char *fmt, ...) {
  unsigned int *adx;
  int c;
  char *s;

  adx = (unsigned int *) (&fmt + 1);
  while (1) {
    while ((c = *fmt++) != '%') {
      if (c == '\0') {
        return;
      }
      putchar(c);
    }
    c = *fmt++;
    if (c == 'd' || c == 'u' || c == 'o' || c == 'x') {
      printn((long) *adx, c == 'o' ? 8 : (c == 'x' ? 16 : 10));
    } else
    if (c == 's') {
      s = (char *) *adx;
      while ((c = *s++) != '\0') {
        putchar(c);
      }
    } else
    if (c == 'D') {
      printn(*(long *)adx, 10);
      adx += sizeof(long) / sizeof(int) - 1;
    }
    adx++;
  }
}


/*
 * Print an integer in base b.
 */
void printn(long n , int b) {
  long a;

  if (n < 0) {
    putchar('-');
    n = -n;
  }
  a = n / b;
  if (a != 0) {
    printn(a, b);
  }
  putchar("0123456789ABCDEF"[(int) (n % b)]);
}


/*
 * Panic is called on unresolvable fatal errors.
 * It syncs, prints "panic: mesg" and then loops.
 */
void panic(char *s) {
  update();
  printf("panic: %s\n", s);
  for (;;) {
    idle();
  }
}


/*
 * Prdev prints a warning message of the form
 * "mesg on dev x/y". x and y are the major
 * and minor parts of the device argument.
 */
void prdev(char *str, dev_t dev) {
  printf("%s on dev %u/%u\n", str, major(dev), minor(dev));
}


/*
 * Deverror prints a diagnostic from a device driver.
 * It prints the device, block number, and an octal
 * word (usually some error status register) passed
 * as argument.
 */
void deverror(struct buf *bp, int o1, int o2) {
  prdev("err", bp->b_dev);
  printf("bn=%D er=%o,%o\n", bp->b_blkno, o1, o2);
}
